# RetroTxt - Privacy

#### RetroTxt does not collect or transmit any data created by your web browser.

All data generated by the Extension is only ever stored locally on your computer by using the browser local or session storage APIs.

## Technical

The Extension [manifest.json](../blob/master/manifest.json) lists all the browser permissions RetroTxt requires for operation.

- `permissions` lists required permissions.
- `optional_permissions` lists optional permissions.

## RetroTxt gets these permissions automatically on installation.

<a id="activeTab">

### `activeTab`

It is needed so RetroTxt can read the current URL of the active browser tab. The URL is used to determine:

- If the URL is known to the _Run RetroTxt on files hosted on these domains_ list.
- If the URL matches `*://retrotxt.com/*`.
- If the file it is pointing to is a known text filename, it uses an extension such as `.nfo` or `.txt`.

The read URLs are never saved or transmitted, and the code for their use can be found and reviewed in `scripts\eventpage.js` under the
`_checkURL()`, and `compatibleURL()` functions within the `class Tab`.

### `contextMenus`

Grants RetroTxt access to the contextMenus API, and allows it to create menus on the browser page and RetroTxt toolbar icon.

### `storage`

Grants RetroTxt access to the Storage API, and allows it to save and retrieve user Options configurations.

<a id="retrotxt.com">

### `*://retrotxt.com/*`

Grants RetroTxt access to parse the sample and test text files hosted on retrotxt.com.

## Optional permissions requests

These permissions toggle when you enable specific RetroTxt Options. The browser will prompt you for a permissions request, and if you deny this, the feature will remain off.

### `tabs`

It is needed so RetroTxt can read the URL of background browser tabs for the same purpose as the <a href="#activeTab">`activeTab` permission</a>.

### `downloads`<br>`downloads.open`

Are needed by the _Apply RetroTxt to any downloaded or local text files_. RetroTxt ignores all downloads except those which match the following conditions.

- The file has a MIME type of either `text/plain`, `text/x-nfo`, `text-unknown`.
- The text file content does not begin with the characters `<!` or `<?`, usually HTML or scripts.

The code can be found and reviewed in `scripts\eventpage.js` under `class Downloads`.

### `file:///*/`

The _Use RetroTxt as a local text file viewer_ option is needed, so RetroTxt can run on text files stored on the host computer hard drive. RetroTxt ignores all files except those which match the following conditions.

- The tab URI is not a directory.
- The filename contains one of these file extensions, `asc`, `ascii`, `ans`, `ansi`, `diz`, `faq`, `nfo`, `pcb`, `text`, `txt`.

The code can be found and reviewed in `scripts\eventpage.js` under the `compatibleURL()` function within the `class Tab`.

### `*://*.examples.com/*`

The other domains listed in `optional_permissions` are for the _Run RetroTxt on files hosted on these domains_ option,
and they operate in the same way as the <a href="#retrotxt.com">`*://retrotxt.com/*` permission</a>.
